package controllers

import (
	"fmt"
	"github.com/kataras/iris"
	"lapp_-wy/utils"
	"lapp_-wy/web/middleware/glog"
	"lapp_-wy/web/middleware/jwts"
	"lapp_-wy/web/models"
	"lapp_-wy/web/supports"
	"time"
)

func GetChargetabList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargetabList"
	logs.Message = "收费记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.Chargetab
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	data.Chargetype, _ = ctx.URLParamInt("chargetype")
	buildingid := ctx.URLParam("buildingid")
	unit := ctx.URLParam("unit")
	room := ctx.URLParam("room")
	data.Chargestartdate = ctx.URLParam("startdate")
	data.Chargeenddate = ctx.URLParam("enddate")
	data.Propertyid = ctx.URLParam("propertyid")
	result, count, err := data.GetPage(pageSize, pageIndex, buildingid, unit, room)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["user"] = user.Role
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetChargetab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargenr, _ = ctx.URLParamInt("chargenr")
	result, err := me.SelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//按照年统计
func GetChargetabByYear(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	beginyear := ctx.URLParam("startYear")
	endyear := ctx.URLParam("endYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	result, count, err := me.ChargeTotalByYear(beginyear, endyear, settledate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	supports.Ok(ctx, supports.OptionSuccess, res)
}

//按楼栋统计
func GetChargetabByBuilding(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	buildingid := ctx.URLParam("building")
	beginyear := ctx.URLParam("startYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	endyear := utils.ValueToInt(beginyear, 0) + 1
	result, count, rate, err := me.ChargeTotalByBuilding(buildingid, beginyear, utils.ValueToString(endyear, ""), settledate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["rate"] = rate
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetChargetabListPage(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargetabList"
	logs.Message = "收费记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.Chargetab
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	chargetype, _ := ctx.URLParamInt("chargetype")
	data.Chargetype = chargetype
	switch chargetype {
	case 1:
		data.Propertyid = ctx.URLParam("number")
	case 2:
		data.Accesscardid = ctx.URLParam("number")
	case 3:
		data.Carportid = ctx.URLParam("number")
	}
	searchtime := ctx.URLParam("expireDate")
	startDate := ctx.URLParam("startDate")
	result, count, total, err := data.ListPage(pageSize, pageIndex, startDate, searchtime)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["total"] = total
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetChargetabReadExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargetabList"
	logs.Message = "收费记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.Chargetab
	var err error
	data.Cid = user.Pid
	chargetype, _ := ctx.URLParamInt("chargetype")
	data.Chargetype = chargetype
	searchtime := ctx.URLParam("expireDate")
	startDate := ctx.URLParam("startDate")
	_, filename, err := data.ReadExcel(startDate, searchtime)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func GetChargetabReadBuildingExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var data models.Chargetab
	var err error
	data.Cid = user.Pid
	buildingid := ctx.URLParam("building")
	beginyear := ctx.URLParam("startYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	endyear := utils.ValueToInt(beginyear, 0) + 1
	_, filename, err := data.ReadBuildExcel(buildingid, beginyear, utils.ValueToString(endyear, ""), settledate)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func GetChargetabReadYearExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	beginyear := ctx.URLParam("startYear")
	endyear := ctx.URLParam("endYear")
	accountMonth := ctx.URLParam("accountMonth")
	settledate := utils.ValueToInt(accountMonth, 0)
	_, filename, err := me.ReadYearExcel(beginyear, endyear, settledate)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func GetChargetabReadDataExcel(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var me models.Chargetab
	me.Cid = user.Pid
	me.Chargetype, _ = ctx.URLParamInt("chargetype")
	buildingid := ctx.URLParam("buildingid")
	unit := ctx.URLParam("unit")
	room := ctx.URLParam("room")
	me.Chargestartdate = ctx.URLParam("startdate")
	me.Chargeenddate = ctx.URLParam("enddate")
	_, filename, err := me.ReadChargeDataExcel(buildingid, unit, room)
	if err != nil {
		fmt.Printf("错误err : %v", err)
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	strUrl := "public/uploadxlsx/" + filename
	supports.Ok(ctx, supports.OptionSuccess, strUrl)
}

func DelChargetab(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "DelChargetab"
	logs.Message = "删除缴费记录"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.Chargetab)
	data.Cid = user.Pid
	data.Chargenr, _ = ctx.URLParamInt("chargenr")
	data.Lastmodifyby = user.Userid
	data.Serialnumber = ctx.URLParam("serialnumber")
	err := data.Del()
	if err != nil {
		glog.InfoExt("删除缴费记录", "错误原因:", err)
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}

	supports.Ok(ctx, "删除成功", "")
}

func GetRecordLogList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetRecordLogList"
	logs.Message = "删除记录列表查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.RecordLog
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	data.Createby = ctx.URLParam("createby")
	startdate := ctx.URLParam("startdate")
	enddate := ctx.URLParam("enddate")

	result, count, err := data.GetPage(pageSize, pageIndex, startdate, enddate)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

// 获取当前和陈欠的缴费统计数据
func GetChargeStatisticData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargeStatisticData"
	logs.Message = "当年及陈欠缴费统计查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var chargeTab models.Chargeallocationtab
	//chargeTab.Cid = user.Pid
	year := time.Now().Year()
	start := fmt.Sprintf("%d0101", year)
	end := fmt.Sprintf("%d1231", year)
	selectType := ctx.URLParam("type")
	result, err := chargeTab.GetChargeStatisticData(cid, year, start, end, selectType)
	if err != nil {
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

// 按日期和缴费类型获取缴费数据
func GetChargeTypeData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargeTypeData"
	logs.Message = "按日期和收款类型查看缴费数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var pageSize = 10
	var pageIndex = 1
	if size := ctx.URLParam("size"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("index"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	date := ctx.URLParam("date")
	tDate, err1 := utils.TimeParseyyyyMMdd(date)
	if err1 != nil {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	// 如为传入或传入的chargeType 不合规则使用-1 进行所有chargeType的分类统计
	date = utils.TimeFormat(tDate, "yyyy-MM-dd")
	chargeWay := ctx.URLParam("charge_way")
	if chargeWay == "" {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	var charge models.Chargetab
	result, err := charge.GetChargeTypeData(cid, chargeWay, date, pageIndex, pageSize)
	if err != nil {
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)

}

// 按日期和缴费类型获取缴费统计数据
func GetChargeArrearageData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Chargetab_controller.go"
	logs.Level = "info"
	logs.Function = "GetChargeArrearageData"
	logs.Message = "按日期查看缴费陈欠数据"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()

	var pageSize = 10
	var pageIndex = 1
	if size := ctx.URLParam("size"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("index"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}

	var cid int
	// 如果公司机关部门可以搜索cid
	if user.Pid == 110 {
		cidStr := ctx.URLParam("cid")
		cid = utils.ValueToInt(cidStr, 0)
	} else {
		cid = user.Pid
	}
	if cid == 0 {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	chargeType, err := ctx.URLParamInt("chargetype")
	if err != nil {
		chargeType = 0
	}
	buildingId := ctx.URLParam("buildingid")
	propertyId := ctx.URLParam("propertyid")
	year := time.Now().Year()
	selectType := ctx.URLParam("type")
	arrearageDate := fmt.Sprintf("%d-12-31", year-1)
	var charge models.Chargetab
	result, err := charge.GetChargeArrearageData(cid, pageIndex, pageSize, year, arrearageDate, selectType, chargeType, buildingId, propertyId)
	if err != nil {
		fmt.Println("outer error:", err)
		supports.Error(ctx, iris.StatusInternalServerError, "抱歉未找到相关信息", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
