package models

import (
	"lapp_-wy/db"
	"lapp_-wy/utils"
	"xorm.io/core"
	"errors"
)

//车位表
type Carporttab struct {
	Cid            int    `json:"cid" xorm:"not null pk INT(4)"`
	Carportid      string `json:"carportid" xorm:"not null pk VARCHAR(100)"`
	Descr          string `json:"descr" xorm:"VARCHAR(255)"`
	Status         int    `json:"status" xorm:"INT(4)"`
	Propertyid     string `json:"propertyid" xorm:"VARCHAR(100)"`
	Propertytypeid string `json:"propertytypeid" xorm:"VARCHAR(100)"`
	Createtime     string `json:"createtime" xorm:"VARCHAR(14)"`
	Lastmodifytime string `json:"lastmodifytime" xorm:"VARCHAR(20)"`
	Lastmodifyby   string `json:"lastmodifyby" xorm:"VARCHAR(20)"`
	Contractid     string `json:"contractid" xorm:"VARCHAR(100)"`
}

func (t *Carporttab) TableName() string {
	return "carporttab"
}

// 清除string字段的右侧空格
func (t *Carporttab) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *Carporttab) Add() error {
	e := db.MasterEngine()
	countrole := new(Carporttab)
	affw, err := e.Table("carporttab").ID(core.PK{t.Cid, t.Carportid}).Count(countrole)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("carporttab").Insert(t)
	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Carporttab) Del() bool {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Cid, t.Carportid}).Delete(&Carporttab{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Carporttab) Update() bool {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Cid, t.Carportid}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *Carporttab) SelectOne() (Carporttab, error) {
	e := db.MasterEngine()
	var data Carporttab
	_, err := e.ID(core.PK{t.Cid, t.Carportid}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//分页
func (t *Carporttab) GetPage(pageSize int, pageIndex int) ([]Carporttab, int, error) {
	data := make([]Carporttab, 0)
	e := db.MasterEngine()

	query := e.Table("carporttab").Where("cid = ? ", t.Cid)
	table := e.Table("carporttab").Where("cid = ? ", t.Cid)
	if !utils.ValueIsEmpty(t.Descr) {
		descr := "%" + t.Descr + "%"
		query = query.And("descr like ?", descr)
		table = table.And("descr like ?", descr)
	}
	Offset := (pageIndex - 1) * pageSize
	err := query.Limit(pageSize,Offset).Desc("createtime").Find(&data)

	pcount := new(Carporttab)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	return data, int(count), nil
}

type CarporttabContract struct {
	Carporttab      `xorm:"extends"`
	Propertytab     `xorm:"extends"`
	Propertytypetab `xorm:"extends"`
	Contracttab     `xorm:"extends"`
}

//车位费查询
func (t *Carporttab) Search(buildingid string, unit string, room string) (ContractInfo, error) {
	//联查
	data := ContractInfo{}
	var info CarporttabContract
	e := db.MasterEngine()
	query := e.Table("carporttab").Join("LEFT", "propertytab", "carporttab.propertyid=propertytab.propertyid and carporttab.cid=propertytab.cid").Join("LEFT", "propertytypetab", "carporttab.propertytypeid=propertytypetab.propertytypeid and carporttab.cid=propertytypetab.cid").Join("LEFT", "contracttab", "carporttab.contractid=contracttab.contractid and carporttab.cid=contracttab.cid").Where("carporttab.cid = ?", t.Cid)
	if !utils.ValueIsEmpty(buildingid){
		query = query.And("propertytab.buildingid = ?",buildingid)
	}
	if !utils.ValueIsEmpty(unit){
		query = query.And("propertytab.unit = ?",unit)
	}
	if !utils.ValueIsEmpty(room){
		query = query.And("propertytab.room = ?",room)
	}
	if !utils.ValueIsEmpty(t.Carportid){
		query = query.And("carporttab.carportid = ?",t.Carportid)
	}
	_, err := query.Get(&info)
	if err != nil {
		return data, err
	}
	data.Propertyid = info.Propertytab.Propertyid
	data.Carportid = info.Carporttab.Carportid
	data.Cid = info.Propertytab.Cid
	data.Room = info.Propertytab.Room
	data.Unit = info.Unit
	data.Buildingid = info.Propertytab.Buildingid
	data.Contractid = info.Propertytab.Contractid
	data.Descr = info.Propertytab.Descr
	data.Contact = info.Propertytab.Contact
	data.Phone1 = info.Propertytab.Phone1
	data.Phone2 = info.Propertytab.Phone2
	data.Constructionarea = info.Propertytab.Constructionarea
	data.Unitprice = info.Propertytypetab.Unitprice
	data.Contracttab = info.Contracttab
	data.Chargetype = 3
	return data, err
}
